import sys
from html2excel import ExcelParser

def run():
    if len(sys.argv) == 3:
        file_path = sys.argv[1]
        save_path = sys.argv[2]
        parser = ExcelParser(file_path)
        parser.to_excel(save_path)
    else:
        # First argument for file name, we'll ignore that
        print("Expected 2 arguments. Got {num}".format(num=len(sys.argv)-1))

if __name__ == "__main__":
    run()

# TODO: handle exceptions



from openpyxl import Workbook

from typing import List, Union, Iterator, Tuple
from bs4.element import Tag


class Parser:
    def __init__(self, file_path):
        self.wb = Workbook()
        self.ws = self.wb.active
        self.file_path = file_path

    def read_file(self):
        """
        returns the data contained in a file
        """
        try:
            with open(self.file_path, 'r') as f:
                data = f.read()
            return data
        except:
            raise Exception("Error while reading input file")

    def get_row(self, table: Tag, tags: Union[List, str]) -> Iterator[Tag]:
        row_data = table.find_all(tags)
        for each in row_data:
            yield each

    def pre_validate_and_format(self, i: int, j: int, col: Tag) -> Tuple[int, str]:
        attrs = col.attrs
        end = j
        if "colspan" in attrs:
            colspan = int(attrs.get("colspan", 1))
            end += colspan - 1
            self.ws.merge_cells(start_row=i, end_row=i,
                                start_column=j, end_column=end)
        # TODO: Handle bold, italics and other attributes
        end += 1
        return (end, col.text.strip())

    def write_cell(self, row, col, val) -> None:
        self.ws.cell(row=row, column=col).value = val

    def get_workbook(self) -> Workbook:
        return self.wb

    def save_workbook(self, loc) -> bool:
        try:
            self.wb.save(loc)
            return True
        except:
            return False



from bs4 import BeautifulSoup
from html2excel.base.parser import Parser


class ExcelParser(Parser):
    def __init__(self, file_path: str):
        super().__init__(file_path)

    def to_excel(self, save_file_path: str, ignore_merged_row: bool = True) -> bool:
        # TODO: handle case when rows are merged
        data = self.read_file()
        soup = BeautifulSoup(data, features='html.parser')
        table_data = soup.table
        if table_data is None:
            raise Exception("No table found")
        data_rows = self.get_row(table_data, ["tr"])
        for i, row in enumerate(data_rows, 1):
            columns = self.get_row(row, ["th", "td"])
            next_j = 1
            for j, col in enumerate(columns, 1):
                j = next_j
                next_j, col_data = self.pre_validate_and_format(i, j, col)
                self.write_cell(i, j, col_data)

        self.save_workbook(save_file_path)



from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    readme = f.read()

setup(
    name = "html2excel",
    version = "0.0.4",
    author = "Neema Tsering",
    author_email = "ntvirus333@gmail.com",
    description = ("Convert HTML Table to Excel file"),
    long_description = readme,
    long_description_content_type = "text/markdown",
    install_requires = ['bs4', 'openpyxl'],
    license = "MIT",
    packages = find_packages(),
)